@@ -18,21 +18,21 @@ extern "C" {
  *
  * the version string like "1.2.3"
  */
-#define LIBXSLT_DOTTED_VERSION "1.0.6"
+#define LIBXSLT_DOTTED_VERSION "1.0.7"
 
 /**
  * LIBXSLT_VERSION:
  *
  * the version number: 1.2.3 value is 1002003
  */
-#define LIBXSLT_VERSION 10006
+#define LIBXSLT_VERSION 10007
 
 /**
  * LIBXSLT_VERSION_STRING:
  *
  * the version number string, 1.2.3 value is "1002003"
  */
-#define LIBXSLT_VERSION_STRING "10006"
+#define LIBXSLT_VERSION_STRING "10007"
 
 /**
  * WITH_XSLT_DEBUG:
@@ -65,6 +65,17 @@ extern "C" {
 #define DEBUG_MEMORY_LOCATION
 #endif
 
+/**
+ * WITH_XSLT_DEBUGGER:
+ *
+ * Activate the compilation of the debugger support. Speed penalty
+ * is insignifiant.
+ * On by default unless --without-debugger is passed to configure
+ */
+#if 1
+#define WITH_XSLT_DEBUGGER
+#endif
+
 /**
  * ATTRIBUTE_UNUSED:
  *
