@@ -23,6 +23,9 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_STDARG_H
+#include <stdarg.h>
+#endif
 #include <libxml/xmlmemory.h>
 #include <libxml/debugXML.h>
 #include <libxml/HTMLtree.h>
@@ -53,7 +56,11 @@
 #define gettimeofday(p1,p2)
 #endif /* _MS_VER */
 #else /* WIN32 */
+#if defined(HAVE_SYS_TIME_H)
 #include <sys/time.h>
+#elif defined(HAVE_TIME_H)
+#include <time.h>
+#endif
 #endif /* WIN32 */
 
 #ifndef HAVE_STAT
@@ -67,6 +74,10 @@
 #  endif
 #endif
 
+xmlParserInputPtr xmlNoNetExternalEntityLoader(const char *URL,
+	                                       const char *ID,
+					       xmlParserCtxtPtr ctxt);
+
 static int debug = 0;
 static int repeat = 0;
 static int timing = 0;
@@ -82,129 +93,105 @@ static int html = 0;
 static int xinclude = 0;
 #endif
 static int profile = 0;
-static int nonet;
-static xmlExternalEntityLoader defaultLoader = NULL;
 
-static struct timeval begin, end;
 static const char *params[16 + 1];
 static int nbparams = 0;
 static const char *output = NULL;
 
-#ifdef LIBXML_CATALOG_ENABLED
-static int xsltNoNetExists(const char *URL) {
-#ifdef HAVE_STAT
-    int ret;
-    struct stat info;
-    const char *path;
+/*
+ * Internal timing routines to remove the necessity to have unix-specific
+ * function calls
+ */
 
-    if (URL == NULL)
-	return(0);
+#if defined(HAVE_GETTIMEOFDAY)
+static struct timeval begin, end;
+/*
+ * startTimer: call where you want to start timing
+ */
+static void startTimer(void)
+{
+    gettimeofday(&begin,NULL);
+}
+/*
+ * endTimer: call where you want to stop timing and to print out a
+ *           message about the timing performed; format is a printf
+ *           type argument
+ */
+static void endTimer(const char *format, ...)
+{
+    long msec;
+    va_list ap;
 
-    if (!xmlStrncmp(BAD_CAST URL, BAD_CAST "file://localhost", 16))
-	path = &URL[16];
-    else if (!xmlStrncmp(BAD_CAST URL, BAD_CAST "file:///", 8)) {
-#ifdef _WIN32
-	path = &URL[8];
+    gettimeofday(&end, NULL);
+    msec = end.tv_sec - begin.tv_sec;
+    msec *= 1000;
+    msec += (end.tv_usec - begin.tv_usec) / 1000;
+
+#ifndef HAVE_STDARG_H
+#error "endTimer required stdarg functions"
+#endif
+    va_start(ap, format);
+    vfprintf(stderr,format,ap);
+    va_end(ap);
+
+    fprintf(stderr, " took %ld ms\n", msec);
+}
+#elif defined(HAVE_TIME_H)
+/*
+ * No gettimeofday function, so we have to make do with calling clock.
+ * This is obviously less accurate, but there's little we can do about
+ * that.
+ */
+
+clock_t begin, end;
+static void startTimer(void)
+{
+    begin=clock();
+}
+static void endTimer(char *format, ...)
+{
+    long msec;
+    va_list ap;
+
+    end=clock();
+    msec = ((end-begin) * 1000) / CLOCKS_PER_SEC;
+
+#ifndef HAVE_STDARG_H
+#error "endTimer required stdarg functions"
+#endif
+    va_start(ap, format);
+    vfprintf(stderr,format,ap);
+    va_end(ap);
+    fprintf(stderr, " took %ld ms\n", msec);
+}
 #else
-	path = &URL[7];
+/*
+ * We don't have a gettimeofday or time.h, so we just don't do timing
+ */
+static void startTimer(void)
+{
+  /*
+   * Do nothing
+   */
+}
+static void endTimer(char *format, ...)
+{
+  /*
+   * We cannot do anything because we don't have a timing function
+   */
+#ifdef HAVE_STDARG_H
+    va_start(ap, format);
+    vfprintf(stderr,format,ap);
+    va_end(ap);
+    fprintf(stderr, " was not timed\n", msec);
+#else
+  /* We don't have gettimeofday, time or stdarg.h, what crazy world is
+   * this ?!
+   */
 #endif
-    } else 
-	path = URL;
-    ret = stat(path, &info);
-    if (ret == 0)
-	return(1);
-#endif
-    return(0);
 }
 #endif
 
-static xmlParserInputPtr
-xsltNoNetExternalEntityLoader(const char *URL, const char *ID,
-                               xmlParserCtxtPtr ctxt) {
-    xmlParserInputPtr input = NULL;
-    xmlChar *resource = NULL;
-
-#ifdef LIBXML_CATALOG_ENABLED
-    xmlCatalogAllow pref;
-
-    /*
-     * If the resource doesn't exists as a file,
-     * try to load it from the resource pointed in the catalogs
-     */
-    pref = xmlCatalogGetDefaults();
-
-    if ((pref != XML_CATA_ALLOW_NONE) && (!xsltNoNetExists(URL))) {
-	/*
-	 * Do a local lookup
-	 */
-	if ((ctxt->catalogs != NULL) &&
-	    ((pref == XML_CATA_ALLOW_ALL) ||
-	     (pref == XML_CATA_ALLOW_DOCUMENT))) {
-	    resource = xmlCatalogLocalResolve(ctxt->catalogs,
-					      (const xmlChar *)ID,
-					      (const xmlChar *)URL);
-        }
-	/*
-	 * Try a global lookup
-	 */
-	if ((resource == NULL) &&
-	    ((pref == XML_CATA_ALLOW_ALL) ||
-	     (pref == XML_CATA_ALLOW_GLOBAL))) {
-	    resource = xmlCatalogResolve((const xmlChar *)ID,
-					 (const xmlChar *)URL);
-	}
-	if ((resource == NULL) && (URL != NULL))
-	    resource = xmlStrdup((const xmlChar *) URL);
-
-	/*
-	 * TODO: do an URI lookup on the reference
-	 */
-	if ((resource != NULL) && (!xsltNoNetExists((const char *)resource))) {
-	    xmlChar *tmp = NULL;
-
-	    if ((ctxt->catalogs != NULL) &&
-		((pref == XML_CATA_ALLOW_ALL) ||
-		 (pref == XML_CATA_ALLOW_DOCUMENT))) {
-		tmp = xmlCatalogLocalResolveURI(ctxt->catalogs, resource);
-	    }
-	    if ((tmp == NULL) &&
-		((pref == XML_CATA_ALLOW_ALL) ||
-	         (pref == XML_CATA_ALLOW_GLOBAL))) {
-		tmp = xmlCatalogResolveURI(resource);
-	    }
-
-	    if (tmp != NULL) {
-		xmlFree(resource);
-		resource = tmp;
-	    }
-	}
-    }
-#endif
-    if (resource == NULL)
-	resource = (xmlChar *) URL;
-
-    if (resource != NULL) {
-        if ((!xmlStrncasecmp((const xmlChar *) resource,
-		            (const xmlChar *) "ftp://", 6)) ||
-            (!xmlStrncasecmp((const xmlChar *) resource,
-		            (const xmlChar *) "http://", 7))) {
-	    fprintf(stderr, "Attempt to load network entity %s \n", resource);
-
-	    if (nonet) {
-		if (resource != (xmlChar *) URL)
-		    xmlFree(resource);
-		return(NULL);
-	    }
-	}
-    }
-    if (defaultLoader != NULL) {
-	input = defaultLoader((const char *) resource, ID, ctxt);
-    }
-    if (resource != (xmlChar *) URL)
-	xmlFree(resource);
-    return(input);
-}
-
 static void
 xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
     xmlDocPtr res;
@@ -212,22 +199,15 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
 #ifdef LIBXML_XINCLUDE_ENABLED
     if (xinclude) {
 	if (timing)
-	    gettimeofday(&begin, NULL);
+	    startTimer();
 	xmlXIncludeProcess(doc);
 	if (timing) {
-	    long msec;
-
-	    gettimeofday(&end, NULL);
-	    msec = end.tv_sec - begin.tv_sec;
-	    msec *= 1000;
-	    msec += (end.tv_usec - begin.tv_usec) / 1000;
-	    fprintf(stderr, "XInclude processing %s took %ld ms\n",
-		    filename, msec);
+	    endTimer("XInclude processing %s", filename);
 	}
     }
 #endif
     if (timing)
-	gettimeofday(&begin, NULL);
+        startTimer();
     if (output == NULL) {
 	if (repeat) {
 	    int j;
@@ -255,19 +235,10 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
 	    res = xsltApplyStylesheet(cur, doc, params);
 	}
 	if (timing) {
-	    long msec;
-
-	    gettimeofday(&end, NULL);
-	    msec = end.tv_sec - begin.tv_sec;
-	    msec *= 1000;
-	    msec += (end.tv_usec - begin.tv_usec) / 1000;
 	    if (repeat)
-		fprintf(stderr,
-			"Applying stylesheet %d times took %ld ms\n",
-			repeat, msec);
+		endTimer("Applying stylesheet %d times", repeat);
 	    else
-		fprintf(stderr,
-			"Applying stylesheet took %ld ms\n", msec);
+		endTimer("Applying stylesheet");
 	}
 	xmlFreeDoc(doc);
 	if (res == NULL) {
@@ -285,37 +256,19 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
 #endif
 	    if (cur->methodURI == NULL) {
 		if (timing)
-		    gettimeofday(&begin, NULL);
+		    startTimer();
 		xsltSaveResultToFile(stdout, res, cur);
-		if (timing) {
-		    long msec;
-
-		    gettimeofday(&end, NULL);
-		    msec = end.tv_sec - begin.tv_sec;
-		    msec *= 1000;
-		    msec += (end.tv_usec - begin.tv_usec) / 1000;
-		    fprintf(stderr, "Saving result took %ld ms\n",
-			    msec);
-		}
+		if (timing)
+		    endTimer("Saving result");
 	    } else {
 		if (xmlStrEqual
 		    (cur->method, (const xmlChar *) "xhtml")) {
 		    fprintf(stderr, "non standard output xhtml\n");
 		    if (timing)
-			gettimeofday(&begin, NULL);
+			startTimer();
 		    xsltSaveResultToFile(stdout, res, cur);
-		    if (timing) {
-			long msec;
-
-			gettimeofday(&end, NULL);
-			msec = end.tv_sec - begin.tv_sec;
-			msec *= 1000;
-			msec +=
-			    (end.tv_usec - begin.tv_usec) / 1000;
-			fprintf(stderr,
-				"Saving result took %ld ms\n",
-				msec);
-		    }
+		    if (timing)
+			endTimer("Saving result");
 		} else {
 		    fprintf(stderr,
 			    "Unsupported non standard output %s\n",
@@ -329,17 +282,8 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
 	xmlFreeDoc(res);
     } else {
 	xsltRunStylesheet(cur, doc, params, output, NULL, NULL);
-	if (timing) {
-	    long msec;
-
-	    gettimeofday(&end, NULL);
-	    msec = end.tv_sec - begin.tv_sec;
-	    msec *= 1000;
-	    msec += (end.tv_usec - begin.tv_usec) / 1000;
-	    fprintf(stderr,
-		"Running stylesheet and saving result took %ld ms\n",
-		    msec);
-	}
+	if (timing)
+	    endTimer("Running stylesheet and saving result");
 	xmlFreeDoc(doc);
     }
 }
@@ -365,7 +309,6 @@ static void usage(const char *name) {
     printf("      --param name value : pass a (parameter,value) pair\n");
     printf("            string values must be quoted like \"'string'\"\n");
     printf("      --nonet refuse to fetch DTDs or entities over network\n");
-    printf("      --warnnet warn against fetching over the network\n");
 #ifdef LIBXML_CATALOG_ENABLED
     printf("      --catalogs : use the catalogs from $SGML_CATALOG_FILES\n");
 #endif
@@ -391,7 +334,6 @@ main(int argc, char **argv)
 
     LIBXML_TEST_VERSION
 
-    defaultLoader = xmlGetExternalEntityLoader();
     xmlLineNumbersDefault(1);
 
     if (novalid == 0)
@@ -461,13 +403,9 @@ main(int argc, char **argv)
         } else if ((!strcmp(argv[i], "-norman")) ||
                    (!strcmp(argv[i], "--norman"))) {
             profile++;
-        } else if ((!strcmp(argv[i], "-warnnet")) ||
-                   (!strcmp(argv[i], "--warnnet"))) {
-            xmlSetExternalEntityLoader(xsltNoNetExternalEntityLoader);
         } else if ((!strcmp(argv[i], "-nonet")) ||
                    (!strcmp(argv[i], "--nonet"))) {
-            xmlSetExternalEntityLoader(xsltNoNetExternalEntityLoader);
-            nonet = 1;
+            xmlSetExternalEntityLoader(xmlNoNetExternalEntityLoader);
 #ifdef LIBXML_CATALOG_ENABLED
         } else if ((!strcmp(argv[i], "-catalogs")) ||
                    (!strcmp(argv[i], "--catalogs"))) {
@@ -540,18 +478,10 @@ main(int argc, char **argv)
         }
         if ((argv[i][0] != '-') || (strcmp(argv[i], "-") == 0)) {
             if (timing)
-                gettimeofday(&begin, NULL);
+                startTimer();
 	    style = xmlParseFile((const char *) argv[i]);
-            if (timing) {
-                long msec;
-
-                gettimeofday(&end, NULL);
-                msec = end.tv_sec - begin.tv_sec;
-                msec *= 1000;
-                msec += (end.tv_usec - begin.tv_usec) / 1000;
-                fprintf(stderr, "Parsing stylesheet %s took %ld ms\n",
-                        argv[i], msec);
-            }
+            if (timing) 
+		endTimer("Parsing stylesheet %s", argv[i]);
 	    if (style == NULL) {
 		fprintf(stderr,  "cannot parse %s\n", argv[i]);
 		cur = NULL;
@@ -590,7 +520,7 @@ main(int argc, char **argv)
         for (; i < argc; i++) {
 	    doc = NULL;
             if (timing)
-                gettimeofday(&begin, NULL);
+                startTimer();
 #ifdef LIBXML_HTML_ENABLED
             if (html)
                 doc = htmlParseFile(argv[i], NULL);
@@ -606,16 +536,8 @@ main(int argc, char **argv)
                 fprintf(stderr, "unable to parse %s\n", argv[i]);
                 continue;
             }
-            if (timing) {
-                long msec;
-
-                gettimeofday(&end, NULL);
-                msec = end.tv_sec - begin.tv_sec;
-                msec *= 1000;
-                msec += (end.tv_usec - begin.tv_usec) / 1000;
-                fprintf(stderr, "Parsing document %s took %ld ms\n",
-                        argv[i], msec);
-            }
+            if (timing)
+		endTimer("Parsing document %s", argv[i]);
 	    xsltProcess(doc, cur, argv[i]);
         }
     }
